#pragma once
#ifndef RE_H
#define RE_H

#include "Lexesis/automata.h"

#include <stdexcept>

namespace lxs {
    class RE
    {
        public:
            virtual ~RE() {}
            virtual int toENFA(ENFA& enfa, int attach) = 0;
            virtual std::string toRe() = 0;
    };

    class EmptyRE : public RE
    {
        public:
            EmptyRE() {}
            ~EmptyRE() {}
            virtual int toENFA(ENFA& enfa, int attach);
            virtual std::string toRe();
    };

    class EpsilonRE : public RE
    {
        public:
            EpsilonRE() {}
            ~EpsilonRE() {}
            virtual int toENFA(ENFA& enfa, int attach);
            virtual std::string toRe();
    };

    class SingleRE : public RE
    {
        public:
            SingleRE(char c) : c(c) {}
            ~SingleRE() {}
            virtual int toENFA(ENFA& enfa, int attach);
            virtual std::string toRe();

            char c;
    };

    class ConcatRE : public RE
    {
        public:
            ConcatRE(RE* e, RE* f) : e(e), f(f) {}
            ~ConcatRE() {delete e; delete f;}
            virtual int toENFA(ENFA& enfa, int attach);
            virtual std::string toRe();

            RE* e, *f;
    };

    class StarRE : public RE
    {
        public:
            StarRE(RE* e) : e(e) {}
            ~StarRE() {delete e;}
            virtual int toENFA(ENFA& enfa, int attach);
            virtual std::string toRe();

            RE* e;
    };

    class PlusRE : public RE
    {
        public:
            PlusRE(RE* e, RE* f) : e(e), f(f) {}
            ~PlusRE() {delete e; delete f;}
            virtual int toENFA(ENFA& enfa, int attach);
            virtual std::string toRe();

            RE* e, *f;
    };

    RE* parseRE(std::string& input);

    class SyntaxError : public std::runtime_error
    {
        public:
            SyntaxError(const char* w) : std::runtime_error(w) {}
    };

} //namespace lxs

#endif //RE_H
